@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkHyperTree.h
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 /**
  * @class   vtkHyperTree
  * @brief   A data object structured as a tree.
@@ -53,7 +41,7 @@
  * Attributes are associated with (all) cells, not with points. The
  * attributes that are associated with coarses, it's used for LoD
  * (Level-of-Detail). The attributes on coarse cells can be given by the
- * code or/and computed by the use of a specifc filter exploiting the
+ * code or/and computed by the use of a specific filter exploiting the
  * values from its children (which can be leaves or not).
  *
  * The geometry is implicitly given by the size of the root node on each
@@ -127,7 +115,7 @@
  * This class was modified by Guenole Harel and Jacques-Bernard Lekien 2014
  * This class was modified by Philippe Pebay, 2016
  * Among others, this class was simplified, optimized (memory), documented and
- * completed for to improve IO XML by Jacques-Bernard Lekien 2018-19
+ * completed for to improve IO XML by Jacques-Bernard Lekien 2018-19,24
  * This work was supported by Commissariat a l'Energie Atomique
  * CEA, DAM, DIF, F-91297 Arpajon, France.
  */
@@ -136,11 +124,14 @@
 #define vtkHyperTree_h
 
 #include "vtkCommonDataModelModule.h" // For export macro
+#include "vtkDeprecation.h"           // Include the macros.
 #include "vtkObject.h"
 
 #include <cassert> // Used internally
+#include <limits>  // Used infinity
 #include <memory>  // std::shared_ptr
 
+VTK_ABI_NAMESPACE_BEGIN
 class vtkBitArray;
 class vtkIdList;
 class vtkHyperTreeGridScales;
@@ -164,7 +155,7 @@ struct vtkHyperTreeData
   // Offset start for the implicit global index mapping fixed by
   // SetGlobalIndexStart after create a tree.
   // If you don't choose implicit global index mapping then this
-  // value is -1. Then, you must to describ explicit global index
+  // value is -1. Then, you must to descrieb explicit global index
   // mapping by using then SetGlobalIndexFromLocal for each cell
   // in tree.
   // The extra cost is equivalent to the cost of a field of values
@@ -177,7 +168,6 @@ class VTKCOMMONDATAMODEL_EXPORT vtkHyperTree : public vtkObject
 {
 public:
   vtkTypeMacro(vtkHyperTree, vtkObject);
-
   void PrintSelf(ostream& os, vtkIndent indent) override;
 
   /**
@@ -202,7 +192,7 @@ public:
    * valid level.
    * @param isParent: a binary decomposition tree by level with
    * constraint all describe children. It is useless to declare
-   * all the lastest values to False, especially the last level
+   * all the latest values to False, especially the last level
    * may not be defined.
    * @param isMasked: a binary mask corresponding. It is useless
    * to declare all the latest values to False.
@@ -249,6 +239,8 @@ public:
    * concatenating breadth first order description and mapping of concatenated
    * trees.
    *
+   * @param depthLimiter the depth limiter by `vtkHyperTreeGrid`.
+   *
    * @param inputMask the mask provided by `vtkHyperTreeGrid`.
    *
    * @param numberOfVerticesPerDepth is self explanatory: from depth 0 to the maximum
@@ -273,10 +265,20 @@ public:
    * @warning Masked subtrees of the input are ignored, so the topology of the
    * output tree can differ from the input depending on that.
    */
-  virtual void ComputeBreadthFirstOrderDescriptor(vtkBitArray* inputMask,
+  virtual void ComputeBreadthFirstOrderDescriptor(unsigned int depthLimiter, vtkBitArray* inputMask,
     vtkTypeInt64Array* numberOfVerticesPerDepth, vtkBitArray* descriptor,
     vtkIdList* breadthFirstIdMap) = 0;
 
+  VTK_DEPRECATED_IN_9_4_0(
+    "You must use depthLimiter parameter for transmit the eponymous member of vtkHyperTreeGrid")
+  void ComputeBreadthFirstOrderDescriptor(vtkBitArray* inputMask,
+    vtkTypeInt64Array* numberOfVerticesPerDepth, vtkBitArray* descriptor,
+    vtkIdList* breadthFirstIdMap)
+  {
+    ComputeBreadthFirstOrderDescriptor(std::numeric_limits<unsigned int>::infinity(), inputMask,
+      numberOfVerticesPerDepth, descriptor, breadthFirstIdMap);
+  }
+
   /**
    * Copy the structure by sharing the decomposition description
    * of the tree.
@@ -298,8 +300,16 @@ public:
    * Set/Get tree index in hypertree grid.
    * Services for internal use between hypertree grid and hypertree.
    */
-  void SetTreeIndex(vtkIdType treeIndex) { this->Datas->TreeIndex = treeIndex; }
-  vtkIdType GetTreeIndex() const { return this->Datas->TreeIndex; }
+  void SetTreeIndex(vtkIdType treeIndex)
+  {
+    assert("pre: datas_non_nullptr" && this->Datas != nullptr);
+    this->Datas->TreeIndex = treeIndex;
+  }
+  vtkIdType GetTreeIndex() const
+  {
+    assert("pre: datas_non_nullptr" && this->Datas != nullptr);
+    return this->Datas->TreeIndex;
+  }
   ///@}
 
   /**
@@ -307,6 +317,7 @@ public:
    */
   unsigned int GetNumberOfLevels() const
   {
+    assert("pre: datas_non_nullptr" && this->Datas != nullptr);
     assert("post: result_greater_or_equal_to_one" && this->Datas->NumberOfLevels >= 1);
     return this->Datas->NumberOfLevels;
   }
@@ -314,18 +325,27 @@ public:
   /**
    * Return the number of all vertices (coarse and fine) in the tree.
    */
-  vtkIdType GetNumberOfVertices() const { return this->Datas->NumberOfVertices; }
+  vtkIdType GetNumberOfVertices() const
+  {
+    assert("pre: datas_non_nullptr" && this->Datas != nullptr);
+    return this->Datas->NumberOfVertices;
+  }
 
   /**
    * Return the number of nodes (coarse) in the tree.
    */
-  vtkIdType GetNumberOfNodes() const { return this->Datas->NumberOfNodes; }
+  vtkIdType GetNumberOfNodes() const
+  {
+    assert("pre: datas_non_nullptr" && this->Datas != nullptr);
+    return this->Datas->NumberOfNodes;
+  }
 
   /**
    * Return the number of leaf (fine) in the tree.
    */
   vtkIdType GetNumberOfLeaves() const
   {
+    assert("pre: datas_non_nullptr" && this->Datas != nullptr);
     return this->Datas->NumberOfVertices - this->Datas->NumberOfNodes;
   }
 
@@ -341,7 +361,7 @@ public:
 
   /**
    * Return the number of children per node of the tree.
-   * This value is branchfactoring scale spatial dimenion (f^d).
+   * This value is branchfactoring scale spatial dimension (f^d).
    */
   vtkIdType GetNumberOfChildren() const { return this->NumberOfChildren; }
 
@@ -357,8 +377,8 @@ public:
 
   /**
    * In an hypertree, all cells are the same size by level. This
-   * fonction initialize this cache system is particular used by
-   * the symetric filter.
+   * function initializes this cache system and is particularly
+   * used by the symmetric filter.
    */
   std::shared_ptr<vtkHyperTreeGridScales> InitializeScales(
     const double* scales, bool reinitialize = false) const;
@@ -391,12 +411,12 @@ public:
   }
 
   /**
-   * Return if implicit global index maping has been used.
+   * Return if implicit global index mapping has been used.
    * If true, the initialize has been done by SetGlobalIndexStart (one call
    * by hypertree).
    * If false, the initialize has been done by SetGlobalIndexFromLocal (one
    * call by cell of hypertree).
-   * GetGlobalIndexFromLocel get the good value of global index mapping for
+   * GetGlobalIndexFromLocal get the good value of global index mapping for
    * one cell what ever the initialize metho used.
    */
   virtual bool IsGlobalIndexImplicit() = 0;
@@ -426,7 +446,11 @@ public:
    * Get the start global index for the current tree for implicit global
    * index mapping.
    */
-  vtkIdType GetGlobalIndexStart() const { return this->Datas->GlobalIndexStart; }
+  vtkIdType GetGlobalIndexStart() const
+  {
+    assert("pre: datas_non_nullptr" && this->Datas != nullptr);
+    return this->Datas->GlobalIndexStart;
+  }
 
   /**
    * Set the mapping between a node index in tree and a explicit global
@@ -470,7 +494,7 @@ public:
 
   /**
    * Return if a vertice identified by index in tree as a terminal node.
-   * For this, all childrens mus be all leaves.
+   * For this, all children must be all leaves.
    * \pre not_valid_index
    * \pre not_valid_child_index
    */
@@ -494,9 +518,9 @@ public:
   ///@{
   /**
    * In an hypertree, all cells are the same size by level. This
-   * fonction initializes this cache system is particulary used by
-   * the symetric filter.
-   * Here, you set a scales since extern description (sharing).
+   * function initializes this cache system and is particularly used
+   * by the symmetric filter.
+   * Here, you set 'scales' since extern description (sharing).
    */
   void SetScales(std::shared_ptr<vtkHyperTreeGridScales> scales) const { this->Scales = scales; }
   ///@}
@@ -545,7 +569,7 @@ protected:
   // Storage of pre-computed per-level cell scales
   // In hypertree grid, one description by hypertree.
   // In Uniform hypertree grid, one description by hypertree grid
-  // (all cells, differents hypertree, are identicals by level).
+  // (all cells, different hypertree, are identical by level).
   mutable std::shared_ptr<vtkHyperTreeGridScales> Scales;
 
 private:
@@ -555,4 +579,5 @@ private:
   void operator=(const vtkHyperTree&) = delete;
 };
 
+VTK_ABI_NAMESPACE_END
 #endif
