@@ -5,9 +5,11 @@
 
 /* This header is configured by VTK's build process.  */
 
-/* E.g. on BlueGene and Cray there is no multithreading */
+/* For platforms that don't support Python threads */
 /* #undef VTK_NO_PYTHON_THREADS */
-/* #undef VTK_PYTHON_FULL_THREADSAFE */
+
+/* This adds the locks that allow Python thread concurrency */
+#define VTK_PYTHON_FULL_THREADSAFE
 
 /* Whether the real python debug library has been provided.  */
 /* #undef VTK_WINDOWS_PYTHON_DEBUGGABLE */
@@ -15,6 +17,6 @@
 /* build specific site-packages suffix. This is used to setup Python
  * module paths during initialization.
  */
-#define VTK_PYTHON_SITE_PACKAGES_SUFFIX "lib/python3.12/site-packages"
+#define VTK_PYTHON_SITE_PACKAGES_SUFFIX "lib/python3.13/site-packages"
 
 #endif
